@@ -18,11 +18,14 @@
 #ifndef AIX
 #include <sys/termios.h> /* needed on Solaris 8 */
 #endif
+#include <sys/socket.h>
+#include <netinet/in.h>
 
 #include "mt.h"
 #include "error.h"
 #include "mem.h"
 #include "term.h"
+#include "color.h"
 #include "utils.h"
 #include "help.h"
 #include "globals.h"
@@ -52,11 +55,15 @@ void wrong_key(void)
 	{
 		/* do nothing */
 	}
-	else
-		error_exit("%d is an unknown beep method\n", beep_method);
 	flushinp();
 }
 
+void draw_border(NEWWIN *mywin)
+{
+	wborder(mywin -> win, box_left_side, box_right_side, box_top_side, box_bottom_side,
+			      box_top_left_hand_corner, box_top_right_hand_corner, box_bottom_left_hand_corner, box_bottom_right_hand_corner);
+}
+
 int ask_yes_no(int what_help, NEWWIN *popup)
 {
 	for(;;)
@@ -136,7 +143,7 @@ void draw_line(NEWWIN *win, linepos_t where)
 
 char * edit_string(NEWWIN *win, int win_y, int win_x, int win_width, int max_width, char numbers_only, char *input_string, int what_help, char first_char, history_t *ph, mybool_t *pcase_insensitive)
 {
-	char *string = (char *)mymalloc(max_width + 1, "edit_string: edit buffer");
+	char *string = (char *)mymalloc(max_width + 1, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 	int str_pos = 0, x = 0;
 	int line_width = win_width;
 
@@ -169,7 +176,7 @@ char * edit_string(NEWWIN *win, int win_y, int win_x, int win_width, int max_wid
 		int prev_str_pos = str_pos;
 		int c;
 
-		if (first_char != -1)
+		if (first_char != (char)-1)
 		{
 			c = first_char;
 			first_char = -1;
@@ -342,7 +349,6 @@ char * edit_string(NEWWIN *win, int win_y, int win_x, int win_width, int max_wid
 					force_redraw = 1;
 				}
 			}
-			LOG("backspace: '%s'\n", string);
 			break;
 		case KEY_LEFT:
 			if (x > 0)
@@ -422,7 +428,7 @@ char * edit_string(NEWWIN *win, int win_y, int win_x, int win_width, int max_wid
 		if (str_pos != prev_str_pos || force_redraw)
 		{
 			int loop;
-			char *dummy = mystrdup(&string[str_pos], "edit_string: edit_string");
+			char *dummy = mystrdup(&string[str_pos], __FILE__, __PRETTY_FUNCTION__, __LINE__);
 			dummy[min(strlen(dummy), line_width)] = 0x00;
 			for(loop=strlen(dummy); loop<line_width; loop++)
 				mvwprintw(win -> win, win_y, win_x + loop, " ");
@@ -459,7 +465,7 @@ NEWWIN * create_popup(int n_lines, int n_colls)
 	newwin = mynewwin(n_lines, n_colls, olines, ocols);
 
 	werase(newwin -> win);
-	box(newwin -> win, 0, 0);
+	draw_border(newwin);
 
 	show_panel(newwin -> pwin);
 
@@ -468,12 +474,15 @@ NEWWIN * create_popup(int n_lines, int n_colls)
 
 void delete_popup(NEWWIN *mywin)
 {
-	mydelwin(mywin);
+	if (mywin)
+	{
+		mydelwin(mywin);
 
-	update_panels();
-	doupdate();
+		update_panels();
+		doupdate();
 
-	myfree(mywin);
+		myfree(mywin);
+	}
 }
 
 void mydelwin(NEWWIN *win)
@@ -481,10 +490,10 @@ void mydelwin(NEWWIN *win)
 	bottom_panel(win -> pwin);
 
 	if (ERR == del_panel(win -> pwin))
-		error_exit("mydelwin: del_panel() failed\n");
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "del_panel() failed\n");
 
 	if (ERR == delwin(win -> win))
-		error_exit("mydelwin: delwin() failed\n");
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "delwin() failed\n");
 }
 
 void mydoupdate()
@@ -495,15 +504,15 @@ void mydoupdate()
 
 NEWWIN * mynewwin(int nlines, int ncols, int begin_y, int begin_x)
 {
-	NEWWIN *nwin = (NEWWIN *)mymalloc(sizeof(NEWWIN), "mynewwin: window");
+	NEWWIN *nwin = (NEWWIN *)mymalloc(sizeof(NEWWIN), __FILE__, __PRETTY_FUNCTION__, __LINE__);
 	/*	nwin -> win = subwin(stdscr, nlines, ncols, begin_y, begin_x); */
 	nwin -> win = newwin(nlines, ncols, begin_y, begin_x);
 	if (!nwin -> win)
-		error_exit("mynewwin: failed to create window (subwin) with dimensions %d-%d at offset %d,%d (terminal size: %d,%d)\n", ncols, nlines, begin_x, begin_y, COLS, LINES);
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Failed to create window with dimensions %dx%d at offset %d,%d (terminal size: %d,%d)\n", ncols, nlines, begin_x, begin_y, COLS, LINES);
 
 	nwin -> pwin = new_panel(nwin -> win);
 	if (!nwin -> pwin)
-		error_exit("mynewwin: failed to create panel for new window\n");
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Failed to create panel.\n");
 
 	nwin -> x_off = begin_x;
 	nwin -> y_off = begin_y;
@@ -513,6 +522,9 @@ NEWWIN * mynewwin(int nlines, int ncols, int begin_y, int begin_x)
 	if (bright_colors)
 		wattron(nwin -> win, A_BOLD);
 
+	if (default_bg_color != -1)
+		wbkgdset(nwin -> win, COLOR_PAIR(find_or_init_colorpair(-1, default_bg_color, 1)));
+
 	return nwin;
 }
 
@@ -620,7 +632,7 @@ int find_colorpair(int fgcolor, int bgcolor, char fuzzy)
 
 	if (fuzzy)
 	{
-		for(loop=0; loop<cp.n_def; loop++)
+		for(loop=cp.n_def-1; loop>=0; loop++)
 		{
 			if (cp.fg_color[loop] == fgcolor && cp.bg_color[loop] == -1)
 				return loop;
@@ -665,7 +677,7 @@ int find_or_init_colorpair(int fgcolor, int bgcolor, char ignore_errors)
 				return 0;
 			}
 
-			error_exit("too many (%d) colorpairs defined!\n", cp.n_def);
+			error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Too many (%d) colorpairs defined.\n", cp.n_def);
 		}
 
 		cp.fg_color[cp.n_def] = fgcolor;
@@ -691,7 +703,7 @@ int colorstr_to_nr(char *str)
 	}
 
 	if (use_colors)
-		error_exit("'%s' is not recognized as a color\n", str);
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "'%s' is not recognized as a color\n", str);
 
 	return -1;
 }
@@ -720,7 +732,7 @@ int attrstr_to_nr(char *str)
 		else if (strcmp(str, "blink") == 0)
 			attr |= A_BLINK;
 		else
-			error_exit("'%s' is not recognized as an attribute\n", str);
+			error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "'%s' is not recognized as an attribute.\n", str);
 
 		if (!slash)
 			break;
@@ -797,7 +809,7 @@ char *attr_to_str(int attr)
 	if (attr & A_DIM)
 		attr_to_str_helper(buffer, "dim");
 
-	return mystrdup(buffer, "attr_to_str");
+	return mystrdup(buffer, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 }
 
 void determine_terminal_size(int *max_y, int *max_x)
@@ -866,7 +878,7 @@ int ansi_code_to_color(int value)
 char * emulate_terminal(char *string, color_offset_in_line **cmatches, int *n_cmatches)
 {
 	int len = strlen(string), new_offset = 0, loop;
-	char *new_string = (char *)mymalloc(len + 1, "emulate_terminal: new string");
+	char *new_string = (char *)mymalloc(len + 1, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 	int cur_n_cmatches = 0;
 
 	*n_cmatches = 0;
@@ -897,14 +909,12 @@ char * emulate_terminal(char *string, color_offset_in_line **cmatches, int *n_cm
 
 					string[find_cmd] = 0x00;
 
-					LOG("in: %s\n", p);
 					while(p)
 					{
 						int dummy;
 						char *newp = strchr(p, ';');
 						if (newp) *newp = 0x00;
 
-						LOG("cur: %s\n", p);
 						dummy = atoi(p);
 
 						if (dummy >= 40 && dummy <= 47)
@@ -925,7 +935,6 @@ char * emulate_terminal(char *string, color_offset_in_line **cmatches, int *n_cm
 					}
 
 					ci.colorpair_index = find_or_init_colorpair(fg_i, bg_i, 1);
-					LOG("color: %d %d,%d\n", ci.colorpair_index, fg_i, bg_i);
 					ci.attrs = attr;
 
 					string[find_cmd] = 'm';
@@ -940,7 +949,7 @@ char * emulate_terminal(char *string, color_offset_in_line **cmatches, int *n_cm
 				{
 					cur_n_cmatches = (cur_n_cmatches + 8) * 2;
 
-					*cmatches = (color_offset_in_line *)myrealloc(*cmatches, cur_n_cmatches * sizeof(color_offset_in_line), "emulate_terminal: cmatches");
+					*cmatches = realloc_color_offset_in_line(*cmatches, cur_n_cmatches, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 				}
 
 				memset(&(*cmatches)[*n_cmatches], 0x00, sizeof(color_offset_in_line));
@@ -988,7 +997,6 @@ void init_ncurses(void)
 	refresh();
 	nodelay(stdscr, FALSE);
 	meta(stdscr, TRUE);	/* enable 8-bit input */
-	raw();			/* to be able to catch ctrl+c */
 	idlok(stdscr, TRUE);	/* may give a little clunky screenredraw */
 	idcok(stdscr, TRUE);	/* may give a little clunky screenredraw */
 	leaveok(stdscr, FALSE);
@@ -1004,7 +1012,7 @@ void init_colornames(void)
 	dummy = min(256, COLORS);
 	if (use_colors)
 	{
-		color_names = (char **)mymalloc(dummy * sizeof(char *), "init_colors: color_names");
+		color_names = (char **)mymalloc(dummy * sizeof(char *), __FILE__, __PRETTY_FUNCTION__, __LINE__);
 		memset(color_names, 0x00, dummy * sizeof(char *));
 
 		color_names[COLOR_RED]    = "red";
